@@ -26,9 +26,7 @@
  *  Robert Bragg  <robert@linux.intel.com>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "cogl-util.h"
 #include "cogl-object.h"
@@ -1473,7 +1471,6 @@ void
 cogl_framebuffer_push_path_clip (CoglFramebuffer *framebuffer,
                                  CoglPath *path)
 {
-  CoglClipState *clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
   CoglMatrixEntry *modelview_entry =
     _cogl_framebuffer_get_modelview_entry (framebuffer);
   CoglMatrixEntry *projection_entry =
@@ -1487,8 +1484,8 @@ cogl_framebuffer_push_path_clip (CoglFramebuffer *framebuffer,
       framebuffer->viewport_height
   };
 
-  clip_state->stacks->data =
-    _cogl_clip_stack_push_from_path (clip_state->stacks->data,
+  framebuffer->clip_stack =
+    _cogl_clip_stack_push_from_path (framebuffer->clip_stack,
                                      path,
                                      modelview_entry,
                                      projection_entry,
