@@ -50,6 +50,8 @@
 #include "cogl-config-private.h"
 #include "cogl-error-private.h"
 
+#include "cogl/deprecated/cogl-framebuffer-deprecated.h"
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -79,10 +81,10 @@ static void
 _cogl_init_feature_overrides (CoglContext *ctx)
 {
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_VBOS)))
-    ctx->private_feature_flags &= ~COGL_PRIVATE_FEATURE_VBOS;
+    COGL_FLAGS_SET (ctx->private_features, COGL_PRIVATE_FEATURE_VBOS, FALSE);
 
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_PBOS)))
-    ctx->private_feature_flags &= ~COGL_PRIVATE_FEATURE_PBOS;
+    COGL_FLAGS_SET (ctx->private_features, COGL_PRIVATE_FEATURE_PBOS, FALSE);
 
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_ARBFP)))
     {
@@ -135,8 +137,8 @@ cogl_context_new (CoglDisplay *display,
                   CoglError **error)
 {
   CoglContext *context;
-  uint8_t default_texture_data[] = { 0xff, 0xff, 0xff, 0xff };
-  CoglBitmap *default_texture_bitmap;
+  uint8_t white_pixel[] = { 0xff, 0xff, 0xff, 0xff };
+  CoglBitmap *white_pixel_bitmap;
   const CoglWinsysVtable *winsys;
   int i;
   CoglError *internal_error = NULL;
@@ -179,7 +181,7 @@ cogl_context_new (CoglDisplay *display,
   /* Init default values */
   memset (context->features, 0, sizeof (context->features));
   context->feature_flags = 0;
-  context->private_feature_flags = 0;
+  memset (context->private_features, 0, sizeof (context->private_features));
 
   context->rectangle_state = COGL_WINSYS_RECTANGLE_STATE_UNKNOWN;
 
@@ -219,7 +221,8 @@ cogl_context_new (CoglDisplay *display,
   context->driver_vtable = display->renderer->driver_vtable;
   context->texture_driver = display->renderer->texture_driver;
 
-  context->private_feature_flags |= display->renderer->private_feature_flags;
+  for (i = 0; i < G_N_ELEMENTS (context->private_features); i++)
+    context->private_features[i] |= display->renderer->private_features[i];
 
   winsys = _cogl_context_get_winsys (context);
   if (!winsys->context_init (context, error))
@@ -284,7 +287,7 @@ cogl_context_new (CoglDisplay *display,
   context->texture_units =
     g_array_new (FALSE, FALSE, sizeof (CoglTextureUnit));
 
-  if ((context->private_feature_flags & COGL_PRIVATE_FEATURE_ANY_GL))
+  if (_cogl_has_private_feature (context, COGL_PRIVATE_FEATURE_ANY_GL))
     {
       /* See cogl-pipeline.c for more details about why we leave texture unit 1
        * active by default... */
@@ -396,7 +399,7 @@ cogl_context_new (CoglDisplay *display,
   context->blit_texture_pipeline = NULL;
 
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES)
-  if ((context->private_feature_flags & COGL_PRIVATE_FEATURE_ALPHA_TEST))
+  if (_cogl_has_private_feature (context, COGL_PRIVATE_FEATURE_ALPHA_TEST))
     /* The default for GL_ALPHA_TEST is to always pass which is equivalent to
      * the test being disabled therefore we assume that for all drivers there
      * will be no performance impact if we always leave the test enabled which
@@ -428,41 +431,49 @@ cogl_context_new (CoglDisplay *display,
   _cogl_matrix_entry_cache_init (&context->builtin_flushed_projection);
   _cogl_matrix_entry_cache_init (&context->builtin_flushed_modelview);
 
-  default_texture_bitmap =
-    cogl_bitmap_new_for_data (context,
-                              1, 1, /* width/height */
-                              COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                              4, /* rowstride */
-                              default_texture_data);
-
   /* Create default textures used for fall backs */
   context->default_gl_texture_2d_tex =
-    cogl_texture_2d_new_from_bitmap (default_texture_bitmap,
-                                     /* internal format */
-                                     COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                     NULL);
+    cogl_texture_2d_new_from_data (context,
+                                   1, 1,
+                                   COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                   0, /* rowstride */
+                                   white_pixel,
+                                   NULL); /* abort on error */
 
   /* If 3D or rectangle textures aren't supported then these will
    * return errors that we can simply ignore. */
   internal_error = NULL;
   context->default_gl_texture_3d_tex =
-    cogl_texture_3d_new_from_bitmap (default_texture_bitmap,
-                                     1, /* height */
-                                     1, /* depth */
-                                     COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                     &internal_error);
+    cogl_texture_3d_new_from_data (context,
+                                   1, 1, 1, /* width, height, depth */
+                                   COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                   0, /* rowstride */
+                                   0, /* image stride */
+                                   white_pixel,
+                                   &internal_error);
   if (internal_error)
     cogl_error_free (internal_error);
 
+  /* TODO: add cogl_texture_rectangle_new_from_data() */
+  white_pixel_bitmap =
+    cogl_bitmap_new_for_data (context,
+                              1, 1, /* width/height */
+                              COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                              4, /* rowstride */
+                              white_pixel);
+
   internal_error = NULL;
   context->default_gl_texture_rect_tex =
-    cogl_texture_rectangle_new_from_bitmap (default_texture_bitmap,
-                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                            &internal_error);
+    cogl_texture_rectangle_new_from_bitmap (white_pixel_bitmap);
+
+  /* XXX: we need to allocate the texture now because the white_pixel
+   * data is on the stack */
+  cogl_texture_allocate (COGL_TEXTURE (context->default_gl_texture_rect_tex),
+                         &internal_error);
   if (internal_error)
     cogl_error_free (internal_error);
 
-  cogl_object_unref (default_texture_bitmap);
+  cogl_object_unref (white_pixel_bitmap);
 
   cogl_push_source (context->opaque_color_pipeline);
 
@@ -479,7 +490,7 @@ cogl_context_new (CoglDisplay *display,
      pipeline to track whether any layers have point sprite coords
      enabled. We don't need to do this for GL3 or GLES2 because point
      sprites are handled using a builtin varying in the shader. */
-  if ((context->private_feature_flags & COGL_PRIVATE_FEATURE_GL_FIXED) &&
+  if (_cogl_has_private_feature (context, COGL_PRIVATE_FEATURE_GL_FIXED) &&
       cogl_has_feature (context, COGL_FEATURE_ID_POINT_SPRITE))
     GE (context, glEnable (GL_POINT_SPRITE));
 
