@@ -29,18 +29,15 @@
 
 #include <glib.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 #define _COGL_MATRIX_DEBUG_PRINT(MATRIX) \
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_MATRICES))) \
     { \
       g_print ("%s:\n", G_STRFUNC); \
-      _cogl_matrix_print (MATRIX); \
+      cogl_debug_matrix_print (MATRIX); \
     }
 
-void
-_cogl_matrix_print (const CoglMatrix *matrix);
-
 void
 _cogl_matrix_prefix_print (const char *prefix, const CoglMatrix *matrix);
 
@@ -48,7 +45,7 @@ void
 _cogl_matrix_init_from_matrix_without_inverse (CoglMatrix *matrix,
                                                const CoglMatrix *src);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_MATRIX_PRIVATE_H */
 
