@@ -181,7 +181,9 @@ _cogl_dispatch_onscreen_cb (CoglContext *context)
   while (!_cogl_list_empty (&context->onscreen_dirty_queue))
     {
       CoglOnscreenQueuedDirty *qe =
-        _cogl_container_of (context->onscreen_dirty_queue.next, qe, link);
+        _cogl_container_of (context->onscreen_dirty_queue.next,
+                            CoglOnscreenQueuedDirty,
+                            link);
 
       _cogl_list_remove (&qe->link);
 
