@@ -61,9 +61,7 @@ struct _CoglTexture2D
 
 CoglTexture2D *
 _cogl_texture_2d_new_from_bitmap (CoglBitmap *bmp,
-                                  CoglPixelFormat internal_format,
-                                  CoglBool can_convert_in_place,
-                                  CoglError **error);
+                                  CoglBool can_convert_in_place);
 
 #if defined (COGL_HAS_EGL_SUPPORT) && defined (EGL_KHR_image_base)
 /* NB: The reason we require the width, height and format to be passed
@@ -82,7 +80,8 @@ CoglTexture2D *
 _cogl_texture_2d_create_base (CoglContext *ctx,
                               int width,
                               int height,
-                              CoglPixelFormat internal_format);
+                              CoglPixelFormat internal_format,
+                              CoglTextureLoader *loader);
 
 void
 _cogl_texture_2d_set_auto_mipmap (CoglTexture *tex,
