@@ -53,11 +53,9 @@ typedef struct _CoglTexture3D CoglTexture3D;
  * @width: width of the texture in pixels.
  * @height: height of the texture in pixels.
  * @depth: depth of the texture in pixels.
- * @internal_format: the #CoglPixelFormat to use for the GPU
- *    storage of the texture.
  *
- * Creates a new #CoglTexture3D texture with the specified dimensions
- * and pixel format.
+ * Creates a low-level #CoglTexture3D texture with the specified
+ * dimensions and pixel format.
  *
  * The storage for the texture is not allocated before this function
  * returns. You can call cogl_texture_allocate() to explicitly
@@ -66,6 +64,11 @@ typedef struct _CoglTexture3D CoglTexture3D;
  * how the texture is going to be used and can optimize how it is
  * allocated.
  *
+ * The texture is still configurable until it has been allocated so
+ * for example you can influence the internal format of the texture
+ * using cogl_texture_set_components() and
+ * cogl_texture_set_premultiplied().
+ *
  * <note>This texture will fail to allocate later if
  * %COGL_FEATURE_ID_TEXTURE_3D is not advertised. Allocation can also
  * fail if the requested dimensions are not supported by the
@@ -79,8 +82,7 @@ CoglTexture3D *
 cogl_texture_3d_new_with_size (CoglContext *context,
                                int width,
                                int height,
-                               int depth,
-                               CoglPixelFormat internal_format);
+                               int depth);
 
 /**
  * cogl_texture_3d_new_from_data:
@@ -89,14 +91,6 @@ cogl_texture_3d_new_with_size (CoglContext *context,
  * @height: height of the texture in pixels.
  * @depth: depth of the texture in pixels.
  * @format: the #CoglPixelFormat the buffer is stored in in RAM
- * @internal_format: the #CoglPixelFormat that will be used for storing
- *    the buffer on the GPU. If %COGL_PIXEL_FORMAT_ANY is given then a
- *    premultiplied format similar to the format of the source data will
- *    be used. The default blending equations of Cogl expect premultiplied
- *    color data; the main use of passing a non-premultiplied format here
- *    is if you have non-premultiplied source data and are going to adjust
- *    the blend mode (see cogl_pipeline_set_blend()) or use the data for
- *    something other than straight blending.
  * @rowstride: the memory offset in bytes between the starts of
  *    scanlines in @data or 0 to infer it from the width and format
  * @image_stride: the number of bytes from one image to the next. This
@@ -107,16 +101,21 @@ cogl_texture_3d_new_with_size (CoglContext *context,
  * @data: pointer the memory region where the source buffer resides
  * @error: A CoglError return location.
  *
- * Creates a new 3D texture and initializes it with @data. The data is
- * assumed to be packed array of @depth images. There can be padding
- * between the images using @image_stride.
+ * Creates a low-level 3D texture and initializes it with @data. The
+ * data is assumed to be packed array of @depth images. There can be
+ * padding between the images using @image_stride.
  *
- * Note that this function will throw a #CoglError if
- * %COGL_FEATURE_ID_TEXTURE_3D is not advertised. It can also fail if the
- * requested dimensions are not supported by the GPU.
+ * <note>This api will always immediately allocate GPU memory for the
+ * texture and upload the given data so that the @data pointer does
+ * not need to remain valid once this function returns. This means it
+ * is not possible to configure the texture before it is allocated. If
+ * you do need to configure the texture before allocation (to specify
+ * constraints on the internal format for example) then you can
+ * instead create a #CoglBitmap for your data and use
+ * cogl_texture_3d_new_from_bitmap().</note>
  *
  * Return value: (transfer full): the newly created #CoglTexture3D or
- *               %NULL if there was an error an an exception will be
+ *               %NULL if there was an error and an exception will be
  *               returned through @error.
  * Since: 1.10
  * Stability: Unstable
@@ -127,7 +126,6 @@ cogl_texture_3d_new_from_data (CoglContext *context,
                                int height,
                                int depth,
                                CoglPixelFormat format,
-                               CoglPixelFormat internal_format,
                                int rowstride,
                                int image_stride,
                                const uint8_t *data,
@@ -138,34 +136,39 @@ cogl_texture_3d_new_from_data (CoglContext *context,
  * @bitmap: A #CoglBitmap object.
  * @height: height of the texture in pixels.
  * @depth: depth of the texture in pixels.
- * @internal_format: the #CoglPixelFormat that will be used for storing
- *    the buffer on the GPU. If %COGL_PIXEL_FORMAT_ANY is given then a
- *    premultiplied format similar to the format of the source data will
- *    be used. The default blending equations of Cogl expect premultiplied
- *    color data; the main use of passing a non-premultiplied format here
- *    is if you have non-premultiplied source data and are going to adjust
- *    the blend mode (see cogl_pipeline_set_blend()) or use the data for
- *    something other than straight blending.
- * @error: A CoglError return location.
  *
- * Creates a new 3D texture and initializes it with the images in
- * @bitmap. The images are assumed to be packed together after one
+ * Creates a low-level 3D texture and initializes it with the images
+ * in @bitmap. The images are assumed to be packed together after one
  * another in the increasing y axis. The height of individual image is
  * given as @height and the number of images is given in @depth. The
  * actual height of the bitmap can be larger than @height × @depth. In
  * this case it assumes there is padding between the images.
  *
- * Return value: (transfer full): the newly created texture or %NULL
- *   if there was an error.
+ * The storage for the texture is not allocated before this function
+ * returns. You can call cogl_texture_allocate() to explicitly
+ * allocate the underlying storage or preferably let Cogl
+ * automatically allocate storage lazily when it may know more about
+ * how the texture is going to be used and can optimize how it is
+ * allocated.
+ *
+ * The texture is still configurable until it has been allocated so
+ * for example you can influence the internal format of the texture
+ * using cogl_texture_set_components() and
+ * cogl_texture_set_premultiplied().
+ *
+ * <note>This texture will fail to allocate later if
+ * %COGL_FEATURE_ID_TEXTURE_3D is not advertised. Allocation can also
+ * fail if the requested dimensions are not supported by the
+ * GPU.</note>
+ *
+ * Return value: (transfer full): a newly created #CoglTexture3D
  * Since: 2.0
  * Stability: unstable
  */
 CoglTexture3D *
 cogl_texture_3d_new_from_bitmap (CoglBitmap *bitmap,
-                                 unsigned int height,
-                                 unsigned int depth,
-                                 CoglPixelFormat internal_format,
-                                 CoglError **error);
+                                 int height,
+                                 int depth);
 
 /**
  * cogl_is_texture_3d:
