@@ -25,7 +25,16 @@
 #ifndef __COGL_H__
 #define __COGL_H__
 
+#ifdef COGL_COMPILATION
+#error "<cogl/cogl.h> shouldn't be included internally"
+#endif
+
+/* Note: When building Cogl .gir we explicitly define
+ * __COGL_H_INSIDE__ */
+#ifndef __COGL_H_INSIDE__
 #define __COGL_H_INSIDE__
+#define __COGL_MUST_UNDEF_COGL_H_INSIDE__
+#endif
 
 #ifdef COGL_ENABLE_EXPERIMENTAL_2_0_API
 #ifndef COGL_ENABLE_EXPERIMENTAL_API
@@ -64,14 +73,23 @@
  * 1.x only api...
  */
 #ifndef COGL_ENABLE_EXPERIMENTAL_2_0_API
-#include <cogl/cogl-clip-state.h>
-#include <cogl/cogl-vertex-buffer.h>
 #include <cogl/cogl-enum-types.h>
-#include <cogl/cogl-fixed.h>
-#include <cogl/cogl-material-compat.h>
-#include <cogl/cogl-shader.h>
+#include <cogl/deprecated/cogl-clip-state.h>
+#include <cogl/deprecated/cogl-vertex-buffer.h>
+#include <cogl/deprecated/cogl-fixed.h>
+#include <cogl/deprecated/cogl-material-compat.h>
+#include <cogl/deprecated/cogl-shader.h>
+#include <cogl/deprecated/cogl-texture-deprecated.h>
 #endif
 
+/* It would be good to move these casts up into 1.x only api if we can
+ * update Clutter, Mutter and GnomeShell to avoid redundant casts when
+ * they enable the experimental api... */
+#include <cogl/deprecated/cogl-type-casts.h>
+
+#include <cogl/deprecated/cogl-framebuffer-deprecated.h>
+#include <cogl/deprecated/cogl-auto-texture.h>
+
 /*
  * 2.0 api that's compatible with the 1.x api...
  */
@@ -122,10 +140,7 @@
 #endif
 /* XXX: This will definitly go away once all the Clutter winsys
  * code has been migrated down into Cogl! */
-#include <cogl/cogl-clutter.h>
-#endif
-#ifdef COGL_HAS_SDL_SUPPORT
-#include <cogl/cogl-sdl.h>
+#include <cogl/deprecated/cogl-clutter.h>
 #endif
 
 /*
@@ -138,8 +153,15 @@
  *
  * The cogl_path_ api used to be part of the core Cogl api so for
  * compatability we include cogl-path.h via cogl.h
+ *
+ * Note: we have to make sure not to include cogl-path.h while
+ * building core cogl or generating the Cogl .gir data because
+ * cogl-path now gets built after cogl and some cogl-path headers are
+ * only generated at build time...
  */
-#if defined (COGL_HAS_COGL_PATH_SUPPORT)
+#if defined (COGL_HAS_COGL_PATH_SUPPORT) && \
+  !defined (COGL_COMPILATION) && \
+  !defined (COGL_GIR_SCANNING)
 #include <cogl-path/cogl-path.h>
 #endif
 
@@ -150,6 +172,17 @@
  * General utility functions for COGL.
  */
 
+/* The gobject introspection scanner seems to parse public headers in
+ * isolation which means we need to be extra careful about how we
+ * define and undefine __COGL_H_INSIDE__ used to detect when internal
+ * headers are incorrectly included by developers. In the gobject
+ * introspection case we have to manually define __COGL_H_INSIDE__ as
+ * a commandline argument for the scanner which means we must be
+ * careful not to undefine it in a header...
+ */
+#ifdef __COGL_MUST_UNDEF_COGL_H_INSIDE__
 #undef __COGL_H_INSIDE__
+#undef __COGL_MUST_UNDEF_COGL_H_INSIDE__
+#endif
 
 #endif /* __COGL_H__ */
