@@ -731,426 +731,6 @@ COGL_DEPRECATED_IN_1_16
 void
 cogl_set_source_texture (CoglTexture *texture);
 
-
-/**
- * SECTION:cogl-clipping
- * @short_description: Fuctions for manipulating a stack of clipping regions
- *
- * To support clipping your geometry to rectangles or paths Cogl exposes a
- * stack based API whereby each clip region you push onto the stack is
- * intersected with the previous region.
- */
-
-/**
- * cogl_clip_push_window_rect:
- * @x_offset: left edge of the clip rectangle in window coordinates
- * @y_offset: top edge of the clip rectangle in window coordinates
- * @width: width of the clip rectangle
- * @height: height of the clip rectangle
- *
- * Specifies a rectangular clipping area for all subsequent drawing
- * operations. Any drawing commands that extend outside the rectangle
- * will be clipped so that only the portion inside the rectangle will
- * be displayed. The rectangle dimensions are not transformed by the
- * current model-view matrix.
- *
- * The rectangle is intersected with the current clip region. To undo
- * the effect of this function, call cogl_clip_pop().
- *
- * Deprecated: 1.16: Use cogl_framebuffer_push_scissor_clip() instead
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_push_scissor_clip)
-void
-cogl_clip_push_window_rect (float x_offset,
-                            float y_offset,
-                            float width,
-                            float height);
-
-/**
- * cogl_clip_push_window_rectangle:
- * @x_offset: left edge of the clip rectangle in window coordinates
- * @y_offset: top edge of the clip rectangle in window coordinates
- * @width: width of the clip rectangle
- * @height: height of the clip rectangle
- *
- * Specifies a rectangular clipping area for all subsequent drawing
- * operations. Any drawing commands that extend outside the rectangle
- * will be clipped so that only the portion inside the rectangle will
- * be displayed. The rectangle dimensions are not transformed by the
- * current model-view matrix.
- *
- * The rectangle is intersected with the current clip region. To undo
- * the effect of this function, call cogl_clip_pop().
- *
- * Since: 1.2
- * Deprecated: 1.16: Use cogl_framebuffer_push_scissor_clip() instead
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_push_scissor_clip)
-void
-cogl_clip_push_window_rectangle (int x_offset,
-                                 int y_offset,
-                                 int width,
-                                 int height);
-
-/**
- * cogl_clip_push:
- * @x_offset: left edge of the clip rectangle
- * @y_offset: top edge of the clip rectangle
- * @width: width of the clip rectangle
- * @height: height of the clip rectangle
- *
- * Specifies a rectangular clipping area for all subsequent drawing
- * operations. Any drawing commands that extend outside the rectangle
- * will be clipped so that only the portion inside the rectangle will
- * be displayed. The rectangle dimensions are transformed by the
- * current model-view matrix.
- *
- * The rectangle is intersected with the current clip region. To undo
- * the effect of this function, call cogl_clip_pop().
- *
- * Deprecated: 1.16: The x, y, width, height arguments are inconsistent
- *   with other API that specify rectangles in model space, and when used
- *   with a coordinate space that puts the origin at the center and y+
- *   extending up, it's awkward to use. Please use
- *   cogl_framebuffer_push_rectangle_clip()
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_push_rectangle_clip)
-void
-cogl_clip_push (float x_offset,
-                float y_offset,
-                float width,
-                float height);
-
-/**
- * cogl_clip_push_rectangle:
- * @x0: x coordinate for top left corner of the clip rectangle
- * @y0: y coordinate for top left corner of the clip rectangle
- * @x1: x coordinate for bottom right corner of the clip rectangle
- * @y1: y coordinate for bottom right corner of the clip rectangle
- *
- * Specifies a rectangular clipping area for all subsequent drawing
- * operations. Any drawing commands that extend outside the rectangle
- * will be clipped so that only the portion inside the rectangle will
- * be displayed. The rectangle dimensions are transformed by the
- * current model-view matrix.
- *
- * The rectangle is intersected with the current clip region. To undo
- * the effect of this function, call cogl_clip_pop().
- *
- * Since: 1.2
- * Deprecated: 1.16: Use cogl_framebuffer_push_rectangle_clip()
- *                   instead
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_push_rectangle_clip)
-void
-cogl_clip_push_rectangle (float x0,
-                          float y0,
-                          float x1,
-                          float y1);
-
-/**
- * cogl_clip_push_primitive:
- * @primitive: A #CoglPrimitive describing a flat 2D shape
- * @bounds_x1: x coordinate for the top-left corner of the primitives
- *             bounds
- * @bounds_y1: y coordinate for the top-left corner of the primitives
- *             bounds
- * @bounds_x2: x coordinate for the bottom-right corner of the primitives
- *             bounds
- * @bounds_y2: y coordinate for the bottom-right corner of the
- *             primitives bounds.
- *
- * Sets a new clipping area using a 2D shaped described with a
- * #CoglPrimitive. The shape must not contain self overlapping
- * geometry and must lie on a single 2D plane. A bounding box of the
- * 2D shape in local coordinates (the same coordinates used to
- * describe the shape) must be given. It is acceptable for the bounds
- * to be larger than the true bounds but behaviour is undefined if the
- * bounds are smaller than the true bounds.
- *
- * The primitive is transformed by the current model-view matrix and
- * the silhouette is intersected with the previous clipping area.  To
- * restore the previous clipping area, call
- * cogl_clip_pop().
- *
- * Since: 1.10
- * Stability: unstable
- * Deprecated: 1.16: Use cogl_framebuffer_push_primitive_clip()
- *                   instead
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_push_primitive_clip)
-void
-cogl_clip_push_primitive (CoglPrimitive *primitive,
-                          float bounds_x1,
-                          float bounds_y1,
-                          float bounds_x2,
-                          float bounds_y2);
-
-/**
- * cogl_clip_pop:
- *
- * Reverts the clipping region to the state before the last call to
- * cogl_clip_push().
- *
- * Deprecated: 1.16: Use cogl_framebuffer_pop_clip() instead
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_pop_clip)
-void
-cogl_clip_pop (void);
-
-/**
- * cogl_clip_ensure:
- *
- * Ensures that the current clipping region has been set in GL. This
- * will automatically be called before any Cogl primitives but it
- * maybe be neccessary to call if you are using raw GL calls with
- * clipping.
- *
- * Deprecated: 1.2: Calling this function has no effect
- *
- * Since: 1.0
- */
-COGL_DEPRECATED
-void
-cogl_clip_ensure (void);
-
-/**
- * cogl_clip_stack_save:
- *
- * Save the entire state of the clipping stack and then clear all
- * clipping. The previous state can be returned to with
- * cogl_clip_stack_restore(). Each call to cogl_clip_push() after this
- * must be matched by a call to cogl_clip_pop() before calling
- * cogl_clip_stack_restore().
- *
- * Deprecated: 1.2: This was originally added to allow us to save the
- *   clip stack when switching to an offscreen framebuffer, but it's
- *   not necessary anymore given that framebuffers now own separate
- *   clip stacks which will be automatically switched between when a
- *   new buffer is set. Calling this function has no effect
- *
- * Since: 0.8.2
- */
-COGL_DEPRECATED
-void
-cogl_clip_stack_save (void);
-
-/**
- * cogl_clip_stack_restore:
- *
- * Restore the state of the clipping stack that was previously saved
- * by cogl_clip_stack_save().
- *
- * Deprecated: 1.2: This was originally added to allow us to restore
- *   the clip stack when switching back from an offscreen framebuffer,
- *   but it's not necessary anymore given that framebuffers now own
- *   separate clip stacks which will be automatically switched between
- *   when a new buffer is set. Calling this function has no effect
- *
- * Since: 0.8.2
- */
-COGL_DEPRECATED
-void
-cogl_clip_stack_restore (void);
-
-/**
- * cogl_set_framebuffer:
- * @buffer: A #CoglFramebuffer object, either onscreen or offscreen.
- *
- * This redirects all subsequent drawing to the specified framebuffer. This can
- * either be an offscreen buffer created with cogl_offscreen_new_to_texture ()
- * or in the future it may be an onscreen framebuffers too.
- *
- * Since: 1.2
- * Deprecated: 1.16: The latest drawing apis take explicit
- *                   #CoglFramebuffer arguments so this stack of
- *                   framebuffers shouldn't be used anymore.
- */
-COGL_DEPRECATED_IN_1_16
-void
-cogl_set_framebuffer (CoglFramebuffer *buffer);
-
-/**
- * cogl_push_framebuffer:
- * @buffer: A #CoglFramebuffer object, either onscreen or offscreen.
- *
- * Redirects all subsequent drawing to the specified framebuffer. This can
- * either be an offscreen buffer created with cogl_offscreen_new_to_texture ()
- * or in the future it may be an onscreen framebuffer too.
- *
- * You should understand that a framebuffer owns the following state:
- * <itemizedlist>
- *  <listitem><simpara>The projection matrix</simpara></listitem>
- *  <listitem><simpara>The modelview matrix stack</simpara></listitem>
- *  <listitem><simpara>The viewport</simpara></listitem>
- *  <listitem><simpara>The clip stack</simpara></listitem>
- * </itemizedlist>
- * So these items will automatically be saved and restored when you
- * push and pop between different framebuffers.
- *
- * Also remember a newly allocated framebuffer will have an identity matrix for
- * the projection and modelview matrices which gives you a coordinate space
- * like OpenGL with (-1, -1) corresponding to the top left of the viewport,
- * (1, 1) corresponding to the bottom right and +z coming out towards the
- * viewer.
- *
- * If you want to set up a coordinate space like Clutter does with (0, 0)
- * corresponding to the top left and (framebuffer_width, framebuffer_height)
- * corresponding to the bottom right you can do so like this:
- *
- * |[
- * static void
- * setup_viewport (unsigned int width,
- *                 unsigned int height,
- *                 float fovy,
- *                 float aspect,
- *                 float z_near,
- *                 float z_far)
- * {
- *   float z_camera;
- *   CoglMatrix projection_matrix;
- *   CoglMatrix mv_matrix;
- *
- *   cogl_set_viewport (0, 0, width, height);
- *   cogl_perspective (fovy, aspect, z_near, z_far);
- *
- *   cogl_get_projection_matrix (&amp;projection_matrix);
- *   z_camera = 0.5 * projection_matrix.xx;
- *
- *   cogl_matrix_init_identity (&amp;mv_matrix);
- *   cogl_matrix_translate (&amp;mv_matrix, -0.5f, -0.5f, -z_camera);
- *   cogl_matrix_scale (&amp;mv_matrix, 1.0f / width, -1.0f / height, 1.0f / width);
- *   cogl_matrix_translate (&amp;mv_matrix, 0.0f, -1.0 * height, 0.0f);
- *   cogl_set_modelview_matrix (&amp;mv_matrix);
- * }
- *
- * static void
- * my_init_framebuffer (ClutterStage *stage,
- *                      CoglFramebuffer *framebuffer,
- *                      unsigned int framebuffer_width,
- *                      unsigned int framebuffer_height)
- * {
- *   ClutterPerspective perspective;
- *
- *   clutter_stage_get_perspective (stage, &perspective);
- *
- *   cogl_push_framebuffer (framebuffer);
- *   setup_viewport (framebuffer_width,
- *                   framebuffer_height,
- *                   perspective.fovy,
- *                   perspective.aspect,
- *                   perspective.z_near,
- *                   perspective.z_far);
- * }
- * ]|
- *
- * The previous framebuffer can be restored by calling cogl_pop_framebuffer()
- *
- * Since: 1.2
- * Deprecated: 1.16: The latest drawing apis take explicit
- *                   #CoglFramebuffer arguments so this stack of
- *                   framebuffers shouldn't be used anymore.
- */
-COGL_DEPRECATED_IN_1_16
-void
-cogl_push_framebuffer (CoglFramebuffer *buffer);
-
-/**
- * cogl_pop_framebuffer:
- *
- * Restores the framebuffer that was previously at the top of the stack.
- * All subsequent drawing will be redirected to this framebuffer.
- *
- * Since: 1.2
- * Deprecated: 1.16: The latest drawing apis take explicit
- *                   #CoglFramebuffer arguments so this stack of
- *                   framebuffers shouldn't be used anymore.
- */
-COGL_DEPRECATED_IN_1_16
-void
-cogl_pop_framebuffer (void);
-
-/**
- * cogl_set_draw_buffer:
- * @target: A #CoglBufferTarget that specifies what kind of framebuffer you
- *          are setting as the render target.
- * @offscreen: If you are setting a framebuffer of type COGL_OFFSCREEN_BUFFER
- *             then this is a CoglHandle for the offscreen buffer.
- *
- * Redirects all subsequent drawing to the specified framebuffer. This
- * can either be an offscreen buffer created with
- * cogl_offscreen_new_to_texture () or you can revert to your original
- * on screen window buffer.
- *
- * Deprecated: 1.16: The latest drawing apis take explicit
- *                   #CoglFramebuffer arguments so this stack of
- *                   framebuffers shouldn't be used anymore.
- */
-COGL_DEPRECATED_IN_1_16
-void
-cogl_set_draw_buffer (CoglBufferTarget target,
-                      CoglHandle offscreen);
-
-/**
- * cogl_push_draw_buffer:
- *
- * Save cogl_set_draw_buffer() state.
- *
- * Deprecated: 1.16: The latest drawing apis take explicit
- *                   #CoglFramebuffer arguments so this stack of
- *                   framebuffers shouldn't be used anymore.
- */
-COGL_DEPRECATED_IN_1_16
-void
-cogl_push_draw_buffer (void);
-
-/**
- * cogl_pop_draw_buffer:
- *
- * Restore cogl_set_draw_buffer() state.
- *
- * Deprecated: 1.16: The latest drawing apis take explicit
- *                   #CoglFramebuffer arguments so this stack of
- *                   framebuffers shouldn't be used anymore.
- */
-COGL_DEPRECATED_IN_1_16
-void
-cogl_pop_draw_buffer (void);
-
-/**
- * cogl_read_pixels:
- * @x: The window x position to start reading from
- * @y: The window y position to start reading from
- * @width: The width of the rectangle you want to read
- * @height: The height of the rectangle you want to read
- * @source: Identifies which auxillary buffer you want to read
- *          (only COGL_READ_PIXELS_COLOR_BUFFER supported currently)
- * @format: The pixel format you want the result in
- *          (only COGL_PIXEL_FORMAT_RGBA_8888 supported currently)
- * @pixels: The location to write the pixel data.
- *
- * This reads a rectangle of pixels from the current framebuffer where
- * position (0, 0) is the top left. The pixel at (x, y) is the first
- * read, and the data is returned with a rowstride of (width * 4).
- *
- * Currently Cogl assumes that the framebuffer is in a premultiplied
- * format so if @format is non-premultiplied it will convert it. To
- * read the pixel values without any conversion you should either
- * specify a format that doesn't use an alpha channel or use one of
- * the formats ending in PRE.
- *
- * Deprecated: 1.16: Use cogl_framebuffer_read_pixels() instead
- */
-COGL_DEPRECATED_IN_1_16_FOR (cogl_framebuffer_read_pixels)
-void
-cogl_read_pixels (int x,
-                  int y,
-                  int width,
-                  int height,
-                  CoglReadPixelsFlags source,
-                  CoglPixelFormat format,
-                  uint8_t *pixels);
-
 /**
  * cogl_flush:
  *
