@@ -315,7 +315,9 @@ _cogl_winsys_context_init (CoglContext *context, CoglError **error)
 
   /* We'll manually handle queueing dirty events in response to
    * SDL_VIDEOEXPOSE events */
-  context->private_feature_flags |= COGL_PRIVATE_FEATURE_DIRTY_EVENTS;
+  COGL_FLAGS_SET (context->private_features,
+                  COGL_PRIVATE_FEATURE_DIRTY_EVENTS,
+                  TRUE);
 
   return _cogl_context_update_features (context, error);
 }
